/*
 * (C) Copyright 2004-2009
 * Texas Instruments, <www.ti.com>
 * Richard Woodruff <r-woodruff2@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * ChangeLog:
 *   2012,2013 changed by Sony Corporation
 */

#include <common.h>
#include <asm/arch/cpu.h>
#include <asm/io.h>
#include <asm/arch/bits.h>
#include <asm/arch/mux.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/arch/clocks.h>
#include <asm/arch/mem.h>
#include <i2c.h>
#include <asm/mach-types.h>
#if (CONFIG_COMMANDS & CFG_CMD_NAND) && defined(CFG_NAND_LEGACY)
#include <linux/mtd/nand_legacy.h>
#endif

#define CONFIG_OMAP4_SDC		1


#define GPIO2_IN_DAT	0x48055138
#define GPIO5_IN_DAT	0x4805b138
#define OMAP4430_PRM_RSTST	0x4a307b04
#define OMAP4430_PRM_RSTCTRL	0x4A307B00
#define PRM_RSTCTRL_RESET_WARM_BIT	0x1
#define PUBLIC_SW_BOOT_CFG_ADDR 0x4A326a00
#define PUBLIC_SAR_RAM1_FREE    0x4A326a0c

static u32 get_board_id(void)
{
	int gpio_142;
	int board_id;

	/* clock enable for GPIO5 read */
	sr32(CM_L4PER_GPIO5_CLKCTRL, 0, 32, 0x101);

	/* set mux to gpio for gpio142 */
	MV(CP(UART3_RTS_SD), M3 | IEN | PTD);
	sdelay(100 * 3); /* sdelay = udelay * 2.4 */

	gpio_142 = (readl(GPIO5_IN_DAT) & (1 << (142 - 128))) ? 1 : 0;

	if (gpio_142)
		board_id = 0x0;
	else
		board_id = 0x1;

	return board_id;
}

/*****************************************
 * Routine: board_init
 * Description: Early hardware init.
 *****************************************/
int board_init(void)
{
	int gpio_33;

	struct {
		u32 sig;
		u32 size;
		u16 flag;
		u16 dev1;
		u16 dev2;
		u16 dev3;
		u16 dev4;
		u16 pad;
	} boot_config;
	int i;
	u32 *s = (u32 *)&boot_config;

	/* check cold reset */
	if ((readl(OMAP4430_PRM_RSTST) & 0x1) == 0)
		return 0;

	/* clock enable for gpio33 */
	sr32(CM_L4PER_GPIO2_CLKCTRL, 0, 32, 0x101);

	/* set mux to gpio for gpio33 */
	MV(CP(GPMC_AD9), M3 | IEN | PTU);
	sdelay(100 * 3); /* sdelay = udelay * 2.4 */
	gpio_33 = (readl(GPIO2_IN_DAT) & (1 << (33 - 32))) ? 1 : 0;

	if (gpio_33 == 1)
		return 0;

	boot_config.sig = 0xcf00aa01; /* signature */
	boot_config.size = 12;
	boot_config.flag = 0;
	boot_config.dev1 = 0x45; /* USB */
	boot_config.dev2 = 0x06; /* MMC2(1) */
	boot_config.dev3 = 0x06; /* MMC2(1) */
	boot_config.dev4 = 0x06; /* MMC2(1) */
	boot_config.pad = 0x0;

	writel(PUBLIC_SAR_RAM1_FREE, PUBLIC_SW_BOOT_CFG_ADDR);
	for (i = 0; i < sizeof(boot_config); i += 4)
		writel(s[i / 4], PUBLIC_SW_BOOT_CFG_ADDR + i + 0xc);

	writel(PUBLIC_SAR_RAM1_FREE, PUBLIC_SW_BOOT_CFG_ADDR);

	/* clear cold reset flag */
	writel(0x1, OMAP4430_PRM_RSTST);

	/* warm reset */
	writel(PRM_RSTCTRL_RESET_WARM_BIT, OMAP4430_PRM_RSTCTRL);

	/* not reached */
	return 0;
}

#ifdef ICX_DRAM_TEST

#define MEM_TEST_START	(uint32_t *)0x80000000
#define MEM_TEST_SIZE	0x00002000
#define WDT_WSPR	0x4A314048
#define WDT_WWPS	0x4A314034

int mem_test(void)
{
	volatile uint32_t *p;
	uint32_t r, r1, r2;
	int ret = 0;
	int i;
	extern int serial_switch(int num);

	/* use wmport as debug console */
        MV(CP(MCSPI1_CS1), ( IEN | M1)) /* WM_UART_RXD */ \
        MV(CP(UART3_CTS_RCTX), ( M1)) /* WM_UART_TXD */

	printf("\n");
#ifdef BUILD_DISPLAY_ID
	printf("BUILD_DISPLAY_ID = %s\n", BUILD_DISPLAY_ID);
#endif /* BUILD_DISPLAY_ID */
	printf("MEM_TEST_START = 0x%08x\n", MEM_TEST_START);
	printf("MEM_TEST_SIZE = 0x%08x\n", MEM_TEST_SIZE);

	serial_switch(1);

	printf("\n");
#ifdef BUILD_DISPLAY_ID
	printf("BUILD_DISPLAY_ID = %s\n", BUILD_DISPLAY_ID);
#endif /* BUILD_DISPLAY_ID */
	printf("MEM_TEST_START = 0x%08x\n", MEM_TEST_START);
	printf("MEM_TEST_SIZE = 0x%08x\n", MEM_TEST_SIZE);

	/* normal */
	for (p = MEM_TEST_START; p < MEM_TEST_START + MEM_TEST_SIZE; p++)
		*p = (uint32_t)p;

	for (p = MEM_TEST_START; p < MEM_TEST_START + MEM_TEST_SIZE; p++) {
		r = *p;
		if (r != (uint32_t)p) {
			ret = -1;
			r1 = *p;
			r2 = *p;
			printf("Address-on-data test : %p a %08x e %08x"
				"r1 %08x r2 %08x\n",
				p, r, (uint32_t)p, r1, r2);
		}
	}

	/* inverted */
	for (p = MEM_TEST_START; p < MEM_TEST_START + MEM_TEST_SIZE; p++)
		*p = ~(uint32_t)p;

	for (p = MEM_TEST_START; p < MEM_TEST_START + MEM_TEST_SIZE; p++) {
		r = *p;
		if (r != ~(uint32_t)p) {
			ret = -1;
			r1 = *p;
			r2 = *p;
			printf("Address-on-data test : %p a %08x e %08x"
				"r1 %08x r2 %08x\n",
				p, r, ~(uint32_t)p, r1, r2);
		}
	}

	if (ret)
		printf(">TEST_FAIL<\n");
	else
		printf(">TEST_PASS<\n");

	serial_switch(3);

	if (ret)
		printf(">TEST_FAIL<\n");
	else
		printf(">TEST_PASS<\n");

	/* WDT disable */
	writel(0xaaaa, WDT_WSPR);
	while (readl(WDT_WWPS) & 0x10)
		sdelay(1);
	writel(0x5555, WDT_WSPR);
	while (readl(WDT_WWPS) & 0x10)
		sdelay(1);


	while (1)
		__asm__("wfi");

	/* not reached */
	return 0;
}
#endif /* ICX_DRAM_TEST */

/**********************************************************
 * Routine: set_muxconf_regs
 * Description: Setting up the configuration Mux registers
 *              specific to the hardware. Many pins need
 *              to be moved from protect to primary mode.
 *********************************************************/
void set_muxconf_regs(void)
{
#define BOARD_ID_ADDR 0x4a3273fc
	int board_id;

	board_id = get_board_id();
#ifdef CFG_PRINTF
#if CONFIG_CONS_INDEX == 1
	MV(CP(MCSPI1_CS1), ( IEN| M1)) /* WMPORT_UART_RX */\
	MV(CP(UART3_CTS_RCTX), ( M1)) /* WMPORT_UART_TX */
#elif CONFIG_CONS_INDEX == 3
	MV(CP(UART3_RX_IRRX), ( IEN | M0)) /* DBG_UART_RX */ \
	MV(CP(UART3_TX_IRTX), ( M0)) /* DBG_UART_TX */
#endif
#endif
	writel(board_id, BOARD_ID_ADDR);

	return;
}

/* optionally do something like blinking LED */
void board_hang (void)
{ while (0) {};}
